import enum
import requests
import requests.auth
import argparse
import sys
import getpass
import os


class Format(enum.Enum):
    JSON = ('application/json', '.json')
    XML = ('application/xml', '.xml')

    def __init__(self, mime, ext):
        self.mime = mime
        self.ext = ext


class API:
    def __init__(self, user, password, url_prefix):
        self.credentials = requests.auth.HTTPBasicAuth(user, password)
        self.url_prefix = url_prefix

    @staticmethod
    def _ensure_leading_slash(service):
        if service[0] != '/':
            return '/' + service
        return service

    def _build_service_url(self, service):
        return self.url_prefix + API._ensure_leading_slash(service)

    @staticmethod
    def _build_headers(fmt):
        return {
            'Accept': fmt.mime
        }

    def request_raw_url(self, url, fmt=Format.JSON):
        return requests.get(
            url,
            headers=API._build_headers(fmt),
            auth=self.credentials
        )

    def request_service(self, service, fmt=Format.JSON):
        return self.request_raw_url(self._build_service_url(service), fmt)


class CLI(argparse.ArgumentParser):
    def __init__(self, *args, **kwargs):
        argparse.ArgumentParser.__init__(self, *args, **kwargs)
        self.add_argument('-u', '--user', metavar='USERNAME', type=str, action='store', default=None, dest='user', required=True, help='registered username (required)')
        self.add_argument('-P', '--password-file', metavar='FILE', type=str, action='store', default=None, dest='password_file', help='file containing password (if not specified this script will ask from console)')
        self.add_argument('--url-prefix', metavar='url-prefix', type=str, action='store', default=self.default_url_prefix(), dest='url_prefix', help='BacDive services URL prefix <%(default)s>')
        self.add_argument('-o', '--output-dir', metavar='OUTDIR', type=str, action='store', default='.', dest='outdir', help='path to the output directory <%(default)s>')
        self.format_group = self.add_mutually_exclusive_group()
        self.format_group.add_argument('-j', '--json', action='store_const', const=Format.JSON, dest='fmt', default=Format.JSON, help='download in JSON format <default>')
        self.format_group.add_argument('-x', '--xml', action='store_const', const=Format.XML, dest='fmt', help='download in XML format')
        self.add_argument('terms', metavar='TERM', type=str, action='store', nargs='+', default=[], help='')

    def default_url_prefix(self):
        raise NotImplementedError()

    @staticmethod
    def log(msg, *args):
        sys.stderr.write((msg % args) + '\n')

    def get_args(self):
        args = self.parse_args()
        CLI.log('Server : %s', args.url_prefix)
        if args.password_file is None:
            args.password = getpass.getpass('Password for %s : ' % args.user)
        else:
            with open(args.password_file) as f:
                args.password = ''.join(f).strip()
        os.makedirs(args.outdir, exist_ok=True)
        return args

    @staticmethod
    def _response_or_log(response):
        if response.status_code == 200:
            return response
        CLI.log('SEVERE: request failed (%s) %d %s', response.url, response.status_code, response.reason)
        return None

    @staticmethod
    def _pop_or_None(terms):
        if terms:
            return terms.pop(0)
        return None

    @staticmethod
    def _save_response_content(response, filename):
        if response is not None:
            with open(filename, 'wb') as fd:
                for chunk in response.iter_content(chunk_size=128):
                    fd.write(chunk)
